var path = require('path');
var fs = require('fs');
var express = require('express');
var cors = require('cors');
var proxy = require('express-http-proxy');
var app = express();

var configPath = path.resolve('package.json');
var config = require(configPath).liteProxy;

var corsOptions = {
    origin: function (origin, callback) {
        callback(null, true);
    },
    credentials: true,
};
app.use(cors(corsOptions));

var proxyConfigs = config.proxies;
if (!proxyConfigs) {
    proxyConfigs = [config];
} else {
    var lastProxyConfig = undefined;
    var lastProxyConfigIndex = 0;
    for(var i = 0, iz = proxyConfigs.length; i < iz; ++i) {
        lastProxyConfig = proxyConfigs[i];
        lastProxyConfigIndex = i;
        break;
    }
    proxyConfigs.splice(lastProxyConfigIndex, 1);
    proxyConfigs.push(lastProxyConfig);
}

var corsRestoreHeaders = {};
var corsId = 0;
var corsResponseHeaderKeys = [
    'access-control-allow-origin',
    'access-control-allow-methods',
    'access-control-allow-headers',
    'access-control-max-age',
    'access-control-allow-credentials',
];

proxyConfigs.forEach(function(proxyConfig) {
    var options = {
        proxyReqOptDecorator: function (proxyReqOpts, originalReq) {
            var curLiteProxy = config;
            var curConfig = proxyConfig;
            var configJSONContent = fs.readFileSync(configPath, 'utf8');
            if (configJSONContent) {
                curLiteProxy = JSON.parse(configJSONContent).liteProxy;
                if (curLiteProxy.proxies) {
                    for(var i = 0, iz = curLiteProxy.proxies.length; i < iz; ++i) {
                        if (proxyConfig.path === curLiteProxy.proxies[i].path) {
                            curConfig = curLiteProxy.proxies[i];
                            break;
                        }
                    }
                } else {
                    curConfig = curLiteProxy;
                }
            }
    
            var headers = {};
            if (proxyReqOpts.headers) {
                var originalHeaders = proxyReqOpts.headers;
                for (var key in originalHeaders) {
                    headers[key.toLocaleLowerCase()] = originalHeaders[key];
                }
            }
            headers = Object.assign(headers, curConfig.headers || {});
            
            var reg = new RegExp(curConfig.useOriginalCookiePattern || curLiteProxy.useOriginalCookiePattern || '$^')
            var originalCookie = headers.cookie;
            var useOriginalCookie = true;
            if (!originalCookie || !reg.test(originalCookie)) {
                useOriginalCookie = false;
                headers.cookie = curConfig.cookie || curLiteProxy.cookie || '';
            }
    
            console.log((useOriginalCookie ? '' : 'cookie ') + decodeURIComponent(originalReq.originalUrl));
    
            proxyReqOpts.headers = headers;
            return proxyReqOpts;
        }, 
        filter: function(req, res) {
            res.__cors__id__ = ++corsId;
            var corsHeaders = corsRestoreHeaders[res.__cors__id__] = {};

            corsResponseHeaderKeys.forEach(function(key) {
                var val = res.get(key);
                if (val !== undefined) {
                    corsHeaders[key] = val;
                }
            });
            return true;
        },
        userResHeaderDecorator: function(headers, userReq, userRes, proxyReq, proxyRes) {
            headers = Object.assign({}, headers, corsRestoreHeaders[userRes.__cors__id__]);
            delete corsRestoreHeaders[userRes.__cors__id__];
            return headers;
        },
        https: proxyConfig.https,
    };
    if (proxyConfig.path) {
        app.use(proxyConfig.path, proxy(proxyConfig.host, options));
    } else {
        app.use(proxy(proxyConfig.host, options));
    }
});

var PROXY_PORT = config.port || 8080;
app.listen(PROXY_PORT, function () {
    console.log('Proxy web server listening on port ' + PROXY_PORT);
});